#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

import re

from trac.web.chrome import add_stylesheet, add_script
from trac.core import Component, implements, TracError
from trac.web.api import IRequestHandler

from bhdashboard.api import ILayoutProvider

class JqueryUILayout(Component):
    """Implement some basic jquery UI layouts
    """
    implements(ILayoutProvider, IRequestHandler)

    # ILayoutProvider methods
    def get_layouts(self):
        """Supported layouts.
        """
        yield 'jquerylayout'

    def get_layout_description(self, name):
        return {
                'jquerylayout': "Jquery UI Layout"
            }[name]

    def expand_layout(self, name, context, options):
        """Specify jquery layout template
        """
        req = context.req
        add_stylesheet(req, 'dashboard/css/bootstrap.css')
        add_stylesheet(req, 'dashboard/css/bootstrap-responsive.css')
        add_stylesheet(req, 'bhlabs/css/layout-default.css')
        add_stylesheet(req, 'bhlabs/jquery/css/base/jquery.ui.all.css')
        add_script(req, 'bhlabs/jquery/js/jquery-ui.js')
        add_script(req, 'bhlabs/jquery/js/jquery.layout.js')

        return {'template': 'jq_layout.html'}

    # IRequestHandler methods

    RE_MATCH = re.compile(r'^/jq_layout/(\d+)$')

    def match_request(self, req):
        m = self.RE_MATCH.match(req.path_info)
        if m:
            req.jq_layout_ticket = m.group(1)
            return True

    def process_request(self, req):
        return 'jq_layout_view.html', {'ticket_id' : req.jq_layout_ticket}, None


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2013 Olemis Lang <olemis at gmail.com>
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


r"""Apache(TM) Bloodhound extensions

Web bootstrap handlers powered by Routes middleware.

Copyright 2013 Olemis Lang <olemis at gmail.com>
Licensed under the Apache License
"""

import os.path
from routes import Mapper
from routes.middleware import RoutesMiddleware

from trac.config import ChoiceOption, Option
from trac.core import Component, implements
from trac.env import open_environment
from trac.hooks import BootstrapHandlerBase
from trac.resource import IResourceChangeListener
from trac.web.api import HTTP_STATUS, RequestDone
from trac.web.main import get_environments, RequestWithSession, \
                          send_project_index

from multiproduct.env import ProductEnvironment
from multiproduct.model import Product

__all__ = ['ProductRoutesBootstrapHandler', 'bootstrap_product_subdomain',
           'bootstrap_sibling_products', 'bootstrap_embedded_products',
           'helper_bootstrap_subdomain', 'helper_bootstrap_siblings']


class ProductRoutesBootstrapHandler(BootstrapHandlerBase):
    """Load environment based on Routes middleware.
    """
    @staticmethod
    def VOID_WSGI_APP(environ, start_response):
        return lambda data : None

    def __init__(self, mapper=None):
        """Initialize this instance

        :param mapper: optional instance of `routes.Mapper`
        :param use_method_override: 
        """
        self.mapper = mapper or Mapper()
        self._mw = RoutesMiddleware(self.VOID_WSGI_APP, self.mapper,
                                    use_method_override=False, singleton=False)

    def open_environment(self, environ, start_response):
        """Lookup routes matching the target WSGI environment dictionary.
        Match results will be used as follows:

          - `controller` will be ignored
          - `envname` will be the name of the target Trac environment
            used e.g. to select environment folder when 
            TRAC_ENV_PARENT_DIR is set and also to determine authentication
            domains while running TracStandalone server (but not limited
            to those use cases). This entry will be ignored on single
            environment deployments .
          - `product` target product prefix. Used to load a product environment
            and dispatch the request in product context. If missing or empty
            the target will be the global environment
          - `error_status` HTTP status code returned on error (default=500)
          - `error_reason` HTTP response body on error
          - `action` will be one of the following supported values
            * `dispatch` will trigger request handler dispatching loop
              in the context of the target (Trac | product) environment
            * `envlist` will display environment index page
              ( see wiki:TracInterfaceCustomization#ProjectList ).
              All variables mentioned above will be ignored.
            * `error` will return an HTTP error back to the client
              (see `error_*` variables above)
          - `path_info` : see [http://routes.readthedocs.org/en/latest/setting_up.html#magic-path-info Routes magic path info]

        If no match is detected then it defaults to `{action : 'envlist'}` .
        """
        # Apply routes side-effects
        env_path = environ.get('trac.env_path')
        self._mw(environ, self.VOID_WSGI_APP)
        match = ( environ.get('wsgiorg.routing_args') or 
                  (None, dict(action='envlist')) )[1]

#        log = environ.get('wsgi.errors')
#        if log:
#            log.write("[BH Labs] Routes match " + repr(match))

        action = match.get('action', 'envlist')

        if env_path:
            environ['trac.env_name'] = os.path.basename(env_path)
        elif match:
            env_parent_dir = environ.get('trac.env_parent_dir')
            env_paths = environ.get('trac.env_paths')
            if env_parent_dir or env_paths:
                # The value of 'envname' in routes match is the base name of
                # the environment
                env_name = match.get('envname')

                if action == 'dispatch':
                    if not env_name:
                        self._send_http_error(environ, start_response, 500,
                                              'Missing environment name')
                    # See further processing below
                elif action == 'envlist':
                    # No specific environment requested, so render an 
                    # environment index page
                    send_project_index(environ, start_response, env_parent_dir,
                                       env_paths)
                    raise RequestDone
                elif action == 'error':
                    status = match.get('error_status', 500)
                    try:
                        status = int(status)
                    except ValueError:
                        status = 500
                    self._send_http_error(environ, start_response, 
                                          status, match.get('error_reason'))
                else:
                    self._send_http_error(environ, start_response, 500,
                                          "Unsupported action '%s' in "
                                          "bootstrap routes" % (action,) )

                environ['trac.env_name'] = env_name

                if env_parent_dir:
                    env_path = os.path.join(env_parent_dir, env_name)
                else:
                    env_path = get_environments(environ).get(env_name)

                if not env_path or not os.path.isdir(env_path):
                    errmsg = 'Environment not found'
                    self._send_http_error(environ, start_response,
                                          reason=errmsg)

        if not env_path:
            reason = 'The environment options "TRAC_ENV" or ' \
                     '"TRAC_ENV_PARENT_DIR" or the mod_python ' \
                     'options "TracEnv" or "TracEnvParentDir" are ' \
                     'missing. Trac requires one of these options ' \
                     'to locate the Trac environment(s).'
            self._send_http_error(environ, start_response, 500, reason)

        run_once = environ['wsgi.run_once']

        env = open_environment(env_path, use_cache=not run_once)
        # Ensure env._abs_href is set
        abs_href = env.abs_href
        product_id = match.get('product')
        if product_id:
            try:
                env = ProductEnvironment(env, product_id)
            except LookupError:
                errmsg = "Unknown product '%s'" % (product_id)
                self._send_http_error(environ, start_response, reason=errmsg)
        return env

    def probe_environment(self, environ):
        """Apply routes side-effects upon WSGI `environ`
        """
        # Apply routes side-effects
        env_path = environ.get('trac.env_path')
        self._mw(environ, self.VOID_WSGI_APP)
        match = ( environ.get('wsgiorg.routing_args') or 
                  (None, dict(action='envlist')) )[1]
        action = match.get('action', 'envlist')

        # Update 'trac.env_name'
        if env_path:
            environ['trac.env_name'] = os.path.basename(env_path)
        elif match:
            env_parent_dir = environ.get('trac.env_parent_dir')
            env_paths = environ.get('trac.env_paths')
            if env_parent_dir or env_paths:
                # The value of 'env_name' in routes match is the base name of
                # the environment
                environ['trac.env_name'] = match.get('envname')

    def create_request(self, env, environ, start_response):
        req = RequestWithSession(environ, start_response)
        # Patch needed to be compatible with ProductizedHref
        req.href._global_href = req.href
        req.abs_href._global_href = req.abs_href
        return req

    def _send_http_error(self, environ, start_response, status=404, reason=None):
        status_line = '%s %s' % (status, HTTP_STATUS.get(status, 'Unknown'))
        if not reason:
            reason = '%s (%s)' % (HTTP_STATUS.get(status, 'Unknown'), status)
        write = start_response(status_line,
                       [('Content-Type', 'text/plain'),
                        ('Content-Length', str(len(reason)))])
        write(reason)
        raise RequestDone

    DEFAULT_MATCH_ERROR = dict(error_status=404,
                               error_reason='Not found (no route to target)')


#-------------------------
# Common product resource namespaces
# https://issues.apache.org/bloodhound/wiki/Proposals/BEP-0003#url-mapping
#-------------------------

# Product sub-domain namespace
# https://issues.apache.org/bloodhound/wiki/Proposals/BEP-0003#deploy-domain
# https://issues.apache.org/bloodhound/wiki/Proposals/BEP-0003#deploy-domain-path

def helper_bootstrap_subdomain(b=None, depth=2, global_subdomain=None,
                               base_path=None):
    """Setup Routes bootstrap handler to serve product data in sub-domains.
    Environment names must match target domain name. Product subdomains
    are hosted at the same level in sibling sub-domains.

    :param b:     Optional instance of `ProductRoutesBootstrapHandler`. This
                  parameter serves to the purpose of applying sub-domain routes
                  after connecting an initial set of routes
    :param depth: Optional numeric value used to determine the boundary between
                  domain (i.e. environment) name and sub-domain (i.e. product)
                  name.
                  e.g. `depth=2` for p1.domain.tld, p2.domain.tld, ...
                  e.g. `depth=3` for p1.x.y.tld, p2.x.y.tld, p1.x.z.tld ...
    :param global_subdomain: Optional string value for special sub-domain name
                  for global environment.
                  It may also be a callable object with a signature satisfying
                  the following contract

                  :param raw_env_name: list with matched domain names in order
                  :param product: matched product name (string) or `None`
                  :return: a binary tuple (raw_env_name, product) to override
                           previous match or None to stick with it

    :param base_path: Optional path prefix to Bloodhound URL namespace

    Notice: There is no URL mapping for environments index page
    """
    if depth < 2:
        # At least match top-level and second-level domain identifiers
        raise ValueError("Domains depth must be greater than 2")

    _global_subdomain = None
    if not global_subdomain:
        pass
    elif isinstance(global_subdomain, basestring):
        _global_subdomain = (lambda raw_env_name, product : (raw_env_name, None)
                                if product == global_subdomain else None)
    elif isinstance(global_subdomain, dict):
        _global_subdomain = (lambda raw_env_name, product : (raw_env_name, None)
                                if product == global_subdomain.get(
                                                            tuple(raw_env_name))
                                else None)
    elif callable(global_subdomain):
        def _global_subdomain(raw_env_name, product):
            # FIXME : Paranoia vs performance
            try:
                return global_subdomain(raw_env_name, product)
            except:
                return None

    def sub_domain_condition(environ, result):
        """Extract environment and product name from HTTP `Host` header.
        """
        host = environ.get('HTTP_HOST', '')
        domain_parts = host.split('.')
        if 0 <= len(domain_parts) - depth <= 1:
            env_name = domain_parts[-depth:]
            product = domain_parts[:-depth]
            product = product[0] if product else None
            if _global_subdomain is not None:
                override = _global_subdomain(env_name, product)
                if override is not None:
                    env_name, product = override
            if product:
                result['product'] = product
            result['envname'] = '.'.join(env_name)
            result['action'] = 'dispatch'
        else:
            result['action'] = 'error'
            result['error_status'] = '404'
            result['error_reason'] = 'Unknown host ' + host
        return True

    b = b or ProductRoutesBootstrapHandler()
    base_path = base_path or u''
    b.mapper.connect(None, base_path + u'/{path_info:.*}',
                     conditions={'function' : sub_domain_condition})
    return b

bootstrap_product_subdomain = helper_bootstrap_subdomain()


class ProductDomainRegistrar(Component):
    """Update DNS records for (sub)domains on product creation

    This class is used in combination with bootstrap handlers instantiated by
    `helper_bootstrap_subdomain` e.g. `bootstrap_product_subdomain`
    """
    implements(IResourceChangeListener)

    tsig_key = Option('dns', 'tsig_key', '',
        """DNS server authentication key""")

    tsig_keyname = Option('dns', 'tsig_keyname', '',
        """DNS server authentication key name""")

    dns_server_host = Option('dns', 'host', 'localhost',
        """DNS server host name""")

    dns_type = ChoiceOption('dns', 'record_type', ['CNAME', 'A'],
        """DNS record type i.e. one of `CNAME` (default) or `A`""")

    dns_value = Option('dns', 'record_value', '',
        """DNS record value (defaults to parent domain for CNAME)""")

    # IResourceChangeListener methods

    def match_resource(self, resource):
        """Match product resources
        """
        return isinstance(resource, Product)

    def resource_created(self, resource, context):
        """Create DNS entry for (sub)domain once product is created.
        Parent domain will be retrieved from environment (folder) path name.
        """
        self.log.debug("Attempt DNS zone edit for product %s", resource.prefix)
        try:
            import dns
        except ImportError:
            self.log.error("[DNS] Error: Unable to import module 'dns'")
            return
        if not (self.tsig_key and self.tsig_keyname):
            self.warning('[DNS] Invalid config: DNS server access tokens')
        else:
            global_env = self.env.parent or self.env
            base_domain = os.path.basename(global_env.path)
            dns_value = base_domain \
                        if not self.dns_value and self.dns_type == 'CNAME' \
                        else self.dns_value
            if dns_value:
                subdom = resource.prefix
                try:
                    self._dns_add(self.dns_type, subdom, base_domain, dns_value)
                except:
                    self.log.exception("[DNS] Registration error for %s",
                                       subdom)
                else:
                    self.log.info('[DNS] Domain %s registered successfully',
                                  subdom)
            else:
                self.warning('[DNS] Invalid config: DNS target is empty')

    def resource_changed(self, resource, old_values, context):
        """Do nothing"""

    def resource_deleted(self, resource, context):
        """Do nothing"""

    def resource_version_deleted(self, resource, context):
        """Do nothing"""

    # Internal methods

    def _dns_add(self, dns_type, subdom, base_domain, dns_value, ttl=14400):
        """Add DNS record for product (sub)domain
        """
        import dns.query
        import dns.tsigkeyring
        import dns.update
        keyring = dns.tsigkeyring.from_text({self.tsig_keyname : self.tsig_key})
        update = dns.update.Update(subdom, keyring=keyring)
        update.add(subdom, ttl, dns_type, base_domain + '.')
        response =  dns.query.tcp(update, self.dns_server_host)
        # TODO: Analyze response contents. What if it represents a failure ?


# Two-level product path namespace
# https://issues.apache.org/bloodhound/wiki/Proposals/BEP-0003#deploy-sibling-paths

def helper_bootstrap_siblings(b=None, base_path=None, globalname=u'global',
                              failnomatch=True):
    """Product namespace immediately after environment name

    :param b:          Optional instance of `ProductRoutesBootstrapHandler`.
                       This parameter serves to the purpose of applying product
                       path routes after connecting an initial set of routes
    :param base_path:  Optional path prefix to Bloodhound URL namespace
    :param globalname: Optional string value to reserve an entry for
                       global environment scope. defaults to `'global'`
    :param failnomatch: Setup routes to send an HTTP error back to the client
                       if no match could be made for input URL path.
    """
    b = b or ProductRoutesBootstrapHandler()
    base_path = base_path or u''
    b.mapper.connect(None, u'%s/{envname}/%s/{path_info:.*}' % 
                     (base_path, globalname), action='dispatch')
    b.mapper.connect(None, u'%s/{envname}/{product}/{path_info:.*}' % 
                     (base_path,), action='dispatch')
    b.mapper.connect(None, base_path or u'/', action='envlist')
    if failnomatch:
        # Fallback to error condition if no match
        err = ProductRoutesBootstrapHandler.DEFAULT_MATCH_ERROR.copy()
        update_error_vars = lambda environ, result : result.update(err) or True
        b.mapper.connect('.*', conditions=dict(function=update_error_vars))
    return b

bootstrap_sibling_products = helper_bootstrap_siblings()


# Embedded product path namespace
# Equivalent to default Apache(TM) Bloodhound factories
# https://issues.apache.org/bloodhound/wiki/Proposals/BEP-0003#deploy-global-env-embed

bootstrap_embedded_products = ProductRoutesBootstrapHandler()
bootstrap_embedded_products.mapper.connect(None, 
        u'/{envname}/products/{product}/{path_info:.*}', action='dispatch')
bootstrap_embedded_products.mapper.connect(None, u'/{envname}/{path_info:.*}',
                                           action='dispatch')
bootstrap_embedded_products.mapper.connect(None, u'/', action='envlist')




#  specific language governing permissions and limitations
#  under the License.

import json
import os.path
from trac.attachment import Attachment, AttachmentModule
from trac.core import *

from trac.perm import PermissionError
from trac.resource import get_resource_name, Resource, resource_exists, ResourceNotFound
from trac.timeline.web_ui import TimelineModule
from trac.util import get_reporter_id
from trac.util.datefmt import to_datetime
from trac.util.text import to_unicode
from trac.util.translation import _

from bhlabs import util
from trac.config import ConfigurationError, Option
from trac.web.href import Href
from trac.web.chrome import add_stylesheet, add_script, add_script_data, ITemplateProvider
from trac.resource import Resource
from pkg_resources import get_distribution, resource_filename
from trac.web.api import HTTPBadRequest, IRequestFilter, IRequestHandler,\
ITemplateStreamFilter, RequestDone
from trac.web.chrome import Chrome
import unicodedata

#from bhdashboard.util import dummy_request

class InplaceAttachmentModule(Component):
    """Apache(TM) Bloodhound advanced attachment interface. Enhancements
    include multiple file uploads , drag and drop, and drop zones. The
    solution is powered by jQuery File Upload plugin
    ( https://github.com/blueimp/jQuery-File-Upload )
    """

    implements(IRequestFilter, IRequestHandler, ITemplateProvider)

    # IRequestFilter(Interface):

    def pre_process_request(self, req, handler):
        """ Intercept attachment upload request.
        """
        if handler is AttachmentModule(self.env) and\
           req.args.get('action') == 'inplace':
            return self
        return handler

    def post_process_request(self, req, template, data, content_type):
        """Activate jQuery File Upload form.
        """
        if data is not None:
            data.update({
                'bh_attach_advanced' : True,
                'max_size' : AttachmentModule(self.env).max_size
            })

            #if (data['zurb_attach_advanced']):
            #htdocsdir = req.href.chrome('jqfile') #self.get_htdocs_dirs()

            add_stylesheet(req, 'bhlabs/jquery/css/jquery.fileupload-ui.css')

            add_script(req, 'bhlabs/js/tmpl.js')
            add_script(req, 'bhlabs/js/load-image.js')
            add_script(req, 'bhlabs/js/canvas-to-blob.js')

            #add_script(req, self.jQuery_File_Upload_extras__href('jquery-1.8.2.js'))
            #add_script(req, 'jqfile/jquery/js/jquery-ui.min.js')

            add_script(req, 'bhlabs/jquery.fupload/js/jquery.iframe-transport.min.js')
            add_script(req, 'bhlabs/jquery.fupload/js/jquery.ui.widget.min.js')
            add_script(req, 'bhlabs/jquery.fupload/js/jquery.fileupload.min.js')
            add_script(req, 'bhlabs/jquery.fupload/js/jquery.fileupload-fp.min.js')
            add_script(req, 'bhlabs/jquery.fupload/js/jquery.fileupload-ui.min.js')

            add_stylesheet(req, 'bhlabs/css/jq.css')

        return template, data, content_type


    # IRequestHandler methods

    def match_request(self, req):
        """No need to match request.
        """

    def process_request(self, req):

        """Forward in-place attachment upload request to
        `trac.attachment.AttachmentModule` but return JSON suitable for
        jQuery File Upload processing in the client.

        Note: Only sequential file uploads supported.
        """
        self.log.debug("Processing upload request")
        parent_realm = req.args.get('realm')
        parent_id = req.args.get('id')

        #if not parent_realm or not parent_id:
        if not parent_realm:
            raise HTTPBadRequest(_('Bad request'))

        upload = req.args['attachment']
        if not hasattr(upload, 'filename') or not upload.filename:
            raise HTTPBadRequest(_('No file uploaded'))
        if hasattr(upload.file, 'fileno'):
            size = os.fstat(upload.file.fileno())[6]
        else:
            upload.file.seek(0, 2) # seek to end of file
            size = upload.file.tell()
            upload.file.seek(0)

        # We try to normalize the filename to unicode NFC if we can.
        # Files uploaded from OS X might be in NFD.
        filename = unicodedata.normalize('NFC', unicode(upload.filename, 'utf-8'))
        filename = filename.replace('\\', '/').replace(':', '/')
        filename = os.path.basename(filename)
        if not filename:
            raise HTTPBadRequest(_('No file uploaded'))

        # Default values sent back to the client on error
        result = {
            'author' : get_reporter_id(req, 'author'),
            'name' : filename,
            'size' : size,
            'delete_type' : '',
            'delete_url' : '',
            }


        parent = Resource(parent_realm, parent_id)
        attachment = Attachment(self.env, parent.child('attachment', None))

        fakereq = util.dummy_request(self.env, req.authname)
        fakereq.environ = req.environ

        fakereq.perm = req.perm
        fakereq.args = req.args

        try:
            data = AttachmentModule(self.env)._do_save(fakereq, attachment)
        except RequestDone:
            pass
        except (PermissionError, ResourceNotFound, TracError), exc:
            result['error'] = to_unicode(exc)
        else:
            if data is not None and data.get('is_replace'):
                # Invalid attachment submission
                result['error'] = fakereq.chrome['warnings'][-2]
            else:
                result = {
                    'author' : attachment.author,
                    'name' : attachment.filename,
                    'size' : attachment.size,
                    'url' : req.href.attachment(
                            parent_realm, parent_id, attachment.filename),
                    'download_url' : req.href('raw-attachment',
                            parent_realm, parent_id, attachment.filename),
                    'thumbnail_url' : req.href.chrome('common', 'file.png'),
                    'delete_url' : req.href.attachment(
                            parent_realm, parent_id, attachment.filename,
                            action="delete",
                            __FORM_TOKEN=req.args.get('__FORM_TOKEN')),
                    'delete_type' : 'POST',
                    'desc' : req.args.get('description', ''),
                }

        for header, value in req._inheaders:
            if header == 'accept':
                break
        else:
            value = ''
        if 'application/json' in value:
            ctype = 'application/json'
        else:
            ctype = 'text/plain'
        req.send(json.dumps([result]), ctype, 200)

    # ITemplateProvider methods

    def get_htdocs_dirs(self):
        yield 'bhlabs', resource_filename(__name__, 'htdocs')
    #def get_htdocs_dirs(self):
        #return [('jqfile', resource_filename('jqfile', 'htdocs'))]

    def get_templates_dirs(self):
        return []


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2013 Olemis Lang <olemis at gmail.com>
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


r"""Apache(TM) Bloodhound extensions

This module includes components customizing Apache(TM) Bloodhound
core as well as experimental features.

Copyright 2013 Olemis Lang <olemis at gmail.com>
Licensed under the Apache License
"""
__author__ = 'Olemis Lang'

try:
    # Ignore errors to avoid Internal Server Errors
    from trac.core import TracError
    TracError.__str__ = lambda self: unicode(self).encode('ascii', 'ignore')

#    from bhlabs import *
    msg = 'Ok'
except Exception, exc:
#    raise
    msg = "Exception %s raised: '%s'" % (exc.__class__.__name__, str(exc))



#!/usr/bin/env python

# Copyright 2009-2011 Olemis Lang <olemis at gmail.com>
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from bhlabs.__init__ import __doc__ as DESC

versions = [
    (0, 1, 0),
    ]
    
latest = '.'.join(str(x) for x in versions[-1])

status = {
            'planning' :  "Development Status :: 1 - Planning",
            'pre-alpha' : "Development Status :: 2 - Pre-Alpha",
            'alpha' :     "Development Status :: 3 - Alpha",
            'beta' :      "Development Status :: 4 - Beta",
            'stable' :    "Development Status :: 5 - Production/Stable",
            'mature' :    "Development Status :: 6 - Mature",
            'inactive' :  "Development Status :: 7 - Inactive"
         }
dev_status = status["alpha"]

cats = [
    dev_status,
    
      "Environment :: Plugins", 
      "Environment :: Web Environment", 
      "Framework :: Trac", 
      "Intended Audience :: Developers", 
      "Intended Audience :: Information Technology", 
      "Intended Audience :: Other Audience", 
      "Intended Audience :: System Administrators", 
      "License :: OSI Approved :: Apache Software License", 
      "Operating System :: OS Independent", 
      "Programming Language :: Python", 
      "Programming Language :: Python :: 2.5", 
      "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries", 
      "Topic :: Internet :: WWW/HTTP :: HTTP Servers", 
      "Topic :: Internet :: WWW/HTTP :: WSGI", 
      "Topic :: Software Development :: Bug Tracking", 
      "Topic :: Software Development :: Libraries :: Application Frameworks", 
      "Topic :: Software Development :: Libraries :: Python Modules", 
    ]

# Be compatible with older versions of Python
from sys import version
if version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

# Add the change log to the package description.
chglog = None
try:
    from os.path import dirname, join
    chglog = open(join(dirname(__file__), "CHANGES"))
    DESC+= ('\n\n' + chglog.read())
finally:
    if chglog:
        chglog.close()

DIST_NM = 'BloodhoundLabs'
PKG_INFO = {'bhlabs' : ('bhlabs',                     # Package dir
                            # Package data
                            ['../CHANGES', '../TODO', '../COPYRIGHT', 
                              '../NOTICE', '../README', 'templates/*.html',
                              'htdocs/*.*'],
                          ), 
            }

ENTRY_POINTS = r"""
               [trac.plugins]
               bhlabs = bhlabs
               bhlabs.hooks_routes = bhlabs.hooks_routes
               bhlabs.search = bhlabs.search
               bhlabs.theme = bhlabs.theme
               bhlabs.attachment = bhlabs.attachment
               bhlabs.jquery_layout = bhlabs.jquery_layout
               """

import sys
if sys.version_info[:2] < (2, 7):
    test_deps = ['unittest2']
else:
    test_deps = []

setup(
	name=DIST_NM,
	version=latest,
	description=DESC.split('\n', 1)[0],
	author='Olemis Lang',
	author_email='olemis+trac@gmail.com',
	maintainer='Olemis Lang',
	maintainer_email='olemis+trac@gmail.com',
	url='https://labs.blood-hound.net',
	download_url='http://pypi.python.org/packages/2.5/%s/%s/%s-%s-py2.5.egg' % \
	                              (DIST_NM[0], DIST_NM, DIST_NM, latest,),
	requires = ['trac',
              'multiproduct',
              'routes',
              'themeengine',
#              'dns',
              ],
  install_requires = [
      'setuptools>=0.6b1',
      'Routes>=1.13',
      'WebOb',
      'Trac>=1.0',
      'BloodhoundMultiproduct>=0.6',
      'BloodhoundDashboardPlugin>=0.6',
      # Absolute paths in BloodhoundLabs theme htdocs and templates
      'TracThemeEngine>2.2.1',
#      'dnspython',
  ],
	tests_require = test_deps,
	package_dir = dict([p, i[0]] for p, i in PKG_INFO.iteritems()),
	packages = PKG_INFO.keys(),
	package_data = dict([p, i[1]] for p, i in PKG_INFO.iteritems()),
	include_package_data=True,
	provides = ['%s (%s)' % (p, latest) for p in PKG_INFO.keys()],
	obsoletes = ['%s (>=%s.0.0, <%s)' % (p, versions[-1][0], latest) \
	              for p in PKG_INFO.keys()],
	entry_points = ENTRY_POINTS,
	classifiers = cats,
	long_description= DESC
	)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.


r"""Project dashboard for Apache(TM) Bloodhound

Helper functions and classes.
"""

from functools import update_wrapper
import inspect
from pkg_resources import get_distribution
from urlparse import urlparse
from wsgiref.util import setup_testing_defaults

from trac.core import Component, implements, ExtensionPoint
from trac.util.text import to_unicode
from trac.web.api import Request
from trac.web.chrome import add_link, Chrome
from trac.web.main import RequestDispatcher

#------------------------------------------------------
#    Request handling
#------------------------------------------------------

def dummy_request(env, uname=None):
    environ = {}
    setup_testing_defaults(environ)
    environ.update({
        'REQUEST_METHOD' : 'GET',
        'SCRIPT_NAME' : urlparse(str(env._abs_href())).path,
        'trac.base_url' : str(env._abs_href()),
        })
    req = Request(environ, lambda *args, **kwds: None)
    # Intercept redirection
    req.redirect = lambda *args, **kwds: None
    # Setup user information
    if uname is not None :
        environ['REMOTE_USER'] = req.authname = uname

    rd = RequestDispatcher(env)
    chrome = Chrome(env)
    req.callbacks.update({
        'authname': rd.authenticate,
        'chrome': chrome.prepare_request,
        'hdf': getattr(rd, '_get_hdf', None),
        'lc_time': rd._get_lc_time,
        'locale' : getattr(rd, '_get_locale', None),
        'perm': rd._get_perm,
        'session': rd._get_session,
        'tz': rd._get_timezone,
        'form_token': rd._get_form_token
    })
    return req


#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2013 Olemis Lang <olemis at gmail.com>
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


r"""Apache(TM) Bloodhound extensions

Bloodhound search in product context

Copyright 2013 Olemis Lang <olemis at gmail.com>
Licensed under the Apache License
"""

from trac.core import Component, implements
from trac.web.api import IRequestFilter

from bhsearch.web_ui import BloodhoundSearchModule

from multiproduct.env import ProductEnvironment


__all__ = ['ProductSearchModule',]


class ProductSearchModule(Component):
    """Apache(TM) Bloodhound search in product context.
    """
    implements(IRequestFilter)

    @property
    def in_product_ctx(self):
        return isinstance(self.env, ProductEnvironment)

    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        if isinstance(handler, BloodhoundSearchModule) and self.in_product_ctx:
            return BloodhoundSearchModule(self.env.parent)
        return handler

    def post_process_request(self, req, template, data, content_type):
        return template, data, content_type





#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

import copy
import sys

import pkg_resources

from trac.core import implements
from trac.web.api import IRequestFilter, ITemplateStreamFilter
from trac.web.chrome import INavigationContributor, ITemplateProvider

from themeengine.api import IThemeProvider

from bhtheme.theme import BloodhoundTheme

class BloodhoundLabsTheme(BloodhoundTheme):
    """Advanced Bloodhound theme
    """

    implements(IRequestFilter, INavigationContributor, ITemplateProvider,
               ITemplateStreamFilter)

    BLOODHOUND_TEMPLATE_MAP = copy.deepcopy(BloodhoundTheme.BLOODHOUND_TEMPLATE_MAP)

    BLOODHOUND_TEMPLATE_MAP.update({
            'attachment.html' : ('bh_fu_attachment.html', None),
            'attach_file_form.html' : ('bh_fu_attach_file_form.html', None),
            'list_of_attachments.html' : ('bh_fu_list_of_attachments.html', None),
            'ticket.html': ('bh_fu_ticket.html', None),
            'milestone_view.html': ('bh_fu_milestone_view.html', None),
            'wiki_view.html' : ('bh_fu_wiki_view.html',None)
            ,})

    # IThemeProvider methods

    def get_theme_info(self, name):
        """Reuse Bloodhound theme's info and tweak when necessary
        """
        info = super(BloodhoundLabsTheme, self).get_theme_info('Bloodhound')
        info.update(template=pkg_resources.resource_filename('bhtheme', 
                                                             info['template']),
                    htdocs=pkg_resources.resource_filename('bhtheme', 'htdocs'))
        return info

    def get_template_overrides(name):
        return []

    # ITemplateProvider methods

    def get_htdocs_dirs(self):
        """BloodhoundLabs htdocs
        """
        yield 'bhlabs', pkg_resources.resource_filename(__name__, 'htdocs')

    def get_templates_dirs(self):
        """BloodhoundLabs templates
        """
        yield pkg_resources.resource_filename(__name__, 'templates')

